/*
 * Copyright 2006-2007, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include "sysstat.h"
#include "support.h"

#include <unistd.h>


static void
display_sysstat(sysstat_t *s)
{
	userstat_t *u;
	fsstat_t *fs;
	char Time[32], text[81 * 25 + 1], *t = text,
	     *const e = text + sizeof(text);
	unsigned numuser = 0;

	if (s == 0)
		return;
	t += sprintf(t,s->Hostinfo);
	if (s->Ocpus == s->Ncpus) {
		if (s->Ncpus > 1) {
			t += sprintf(t,", %d CPUs", s->Ocpus);
		} else {
			assert(s->Ocpus == 1);
			t += sprintf(t,", 1 CPU");
		}
	} else {
		assert(s->Ncpus > 1);
		t += sprintf(t,", %d/%d CPUs",s->Ocpus,s->Ncpus);
	}
	if (s->BootTime) {
		int d,h,m;
		clock_t b = s->TS - s->BootTime;
		d = b / (60*60*24);
		b -= d * (60*60*24);
		h = b / (60*60);
		b -= h * (60*60);
		m = b / (60);
		b -= m * (60);
		if (d == 1)
			t += sprintf(t,", up 1 day");
		else if (d)
			t += sprintf(t,", up %d days",d);
		else
			t += sprintf(t,", up %d:%02d:%02d",h,m,b);
	} else if (s->Devices != 1) {
		t += sprintf(t,", %lu disks  ", s->Devices);
	} else {
		t += sprintf(t,", 1 disk  ");
	}
	strftime(Time,sizeof(Time),"%c",localtime(&s->TS));
	t += sprintf(t,"  %.20s\n",Time);
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;

	t += sprintf(t," User   CPU: %s%% | ", per2str(s->UserP));
	t += sprintf(t,"LoadAv/1m:  %4.2f  | ", s->loadavg[0]);
	t += sprintf(t,"Physical:    %s | ", kb2str(s->Physmem));
	if (s->IpInR >= 0)
		t += sprintf(t,"IP rcv/s:   %s \n", dbl2str(s->IpInR));
	else
		t += sprintf(t,"ipacket/s:  %s \n", dbl2str(s->NetInR));

	t += sprintf(t," Kernel CPU: %s%% | ",per2str(s->KernelP));
	t += sprintf(t,"LoadAv/5m:  %4.2f  | ", s->loadavg[1]);
	t += sprintf(t,"Free:        %s | ", kb2str(s->Pagesfree));
	if (s->IpOutR >= 0)
		t += sprintf(t,"IP send/s:  %s \n", dbl2str(s->IpOutR));
	else
		t += sprintf(t,"opacket/s:  %s \n", dbl2str(s->NetOutR));

	t += sprintf(t," Idle   CPU: %s%% | ",per2str(s->IdleP));
	t += sprintf(t,"LoadAv/15m: %4.2f  | ", s->loadavg[2]);
	t += sprintf(t,"Locked:      %s | ", kb2str(s->Pageslocked));
	t += sprintf(t,"net errs: %6lu \n", s->IpInDiscards+s->IpOutDiscards+s->IpInErrs);
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	t += sprintf(t," intr/s:     %s | ", dbl2str(s->IntrR));
	t += sprintf(t,"syscall/s:  %s | ", dbl2str(s->SyscallR));
	t += sprintf(t,"SwapTotal:   %s | ", kb2str(s->SwapTotal));
	t += sprintf(t,"nfsd ops/s: %s \n", dbl2str(s->NfsdR));

	t += sprintf(t," intrblk/s:  %s | ", dbl2str(s->IntrBlkR));
	t += sprintf(t,"sysfork/s:  %s | ", dbl2str(s->SysforkR));
	t += sprintf(t,"SwapFree:    %s | ", kb2str(s->SwapFree));
	t += sprintf(t,"NfsIn/s:    %s \n", kb2str(s->NfsIR));

	t += sprintf(t," xcalls/s:   %s | ", dbl2str(s->XcallsR));
	t += sprintf(t,"sysexec/s:  %s | ", dbl2str(s->SysexecR));
	t += sprintf(t,"scan/s:      %s | ",dbl2str(s->ScanR));
	t += sprintf(t,"NfsOut/s:   %s \n", kb2str(s->NfsOR));

	t += sprintf(t," pswitch/s:  %s | ", dbl2str(s->PswitchR));
	t += sprintf(t,"sysread/s:  %s | ", dbl2str(s->SysreadR));
	if ((s->PgswapinR != 0) && (s->AnonpginR == 0))
		t += sprintf(t,"pgswapin/s:  %s | ", dbl2str(s->PgswapinR));
	else
		t += sprintf(t,"anonpgin/s:  %s | ", dbl2str(s->AnonpginR));
	t += sprintf(t,"DiskIn/s:   %s \n", kb2str(s->DiskIR));

	t += sprintf(t," migrate/s:  %s | ", dbl2str(s->MigrateR));
	t += sprintf(t,"syswrite/s: %s | ", dbl2str(s->SyswriteR));
	if ((s->PgswapoutR != 0) && (s->AnonpgoutR == 0))
		t += sprintf(t,"pgswapout/s: %s | ", dbl2str(s->PgswapoutR));
	else
		t += sprintf(t,"anonpgout/s: %s | ", dbl2str(s->AnonpgoutR));
	t += sprintf(t,"DiskOut/s:  %s \n", kb2str(s->DiskOR));

	t += sprintf(t," invswtch/s: %s | ", dbl2str(s->InvSwtchR));
	t += sprintf(t,"                  | ");
	t += sprintf(t,"anonfree/s   %s | ",dbl2str(s->AnonfreeR));
	//t += sprintf(t,"FsFree:      %s | ", kb2str(s->FsFree));
	t += sprintf(t,"disk errs: %5lu  \n", s->HardErrors+s->SoftErrors);

	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;

	u = s->us;
	while (u) {
		++numuser;
		u = u->next;
	}
	u = s->us;
	t += sprintf(t,"%d",numuser);
	if (u->h_s) {
		if (numuser < 10)
			t += strlcpy(t," User   NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
		else if (numuser < 100)
			t += strlcpy(t," User  NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
		else 
			t += strlcpy(t," User NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS    H+S   %MEM  %CPU   CPU-TIME\n",e - t);
	} else {
		if (numuser < 10)
			t += strlcpy(t," User         NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
		else if (numuser < 100)
			t += strlcpy(t," User        NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
		else 
			t += strlcpy(t," User       NPROC ( O/ S/ R/ Z/ T) NLWP   SIZE   RSS   %MEM  %CPU    CPU-TIME\n",e - t);
	}
	assert(text + sizeof(text) > t);

	while (u) {
		long hr,mn,sc;
		double pctmem;
		if (t + 82 > e) {
			(void) write(STDOUT_FILENO,text,t - text);
			t = text;
		}
		pctmem = (double)u->rss / (double)s->Physmem * 100.0;
		assert(u->time.tv_sec >= 0);
		assert(u->time.tv_nsec >= 0);
		sc = u->time.tv_sec;
		sc += rint((double)u->time.tv_nsec/1000000000);
		mn = sc / 60;
		sc -= mn * 60;
		hr = mn / 60;
		mn -= hr * 60;
		if (u->h_s)
			t += sprintf(t,"%-9s ", u->username);
		else
			t += sprintf(t,"%-15s ", u->username);
		t += sprintf(t,"%4d (%2d/%2d/%2d/%2d/%2d) %4d  %s  ",
				u->nproc,
				u->oncpu,
				u->sleeping,
				u->runnable,
				u->zombie,
				u->stopped,
				u->nlwp,
				kb2str(u->size));
		t += sprintf(t,"%s  ",kb2str(u->rss));
		if (u->h_s)
			t += sprintf(t,"%s  ",kb2str(u->h_s));
		if (u->pctmem < 9.995)
			t += sprintf(t,"%3.1f%%  ",pctmem);
		else
			t += sprintf(t,"%3.f%%  ",pctmem);
		if (u->pctcpu < 9.995)
			t += sprintf(t,"%3.1f%% ",u->pctcpu);
		else
			t += sprintf(t,"%3.f%% ",u->pctcpu);
		t += sprintf(t,"%4d:%02d:%02d\n",hr,mn,sc);
		u = u->next;
	}
	if (t + 82 > e) {
		(void) write(STDOUT_FILENO,text,t - text);
		t = text;
	}
	memcpy(t,"--------------------------------------------------------------------------------\n",81);
	t += 81;
	memcpy(t,"device                typ   mountpoint            ops/s  in/s out/s  avail/ size\n",81);
	t += 81;
	
	for (fs = s->fs; fs; fs = fs->next) {
		const char *f;
		if (t - text + 82 > sizeof(text)) {
			(void) write(STDOUT_FILENO,text,t - text);
			t = text;
		}
		if (strlen(fs->dev) > 21)
			f = "%.18s... %-5s ";
		else
			f = "%-21s %-5s ";
		t += sprintf(t,f,fs->dev,fs->typ);
		if (strlen(fs->mnt) > 21)
			f = "%.18s... %s";
		else
			f = "%-21s %s";
		t += sprintf(t,f,fs->mnt,dbl2str(fs->ops_rate));
		t += sprintf(t," %s/",kb2str(fs->read_rate));
		t += sprintf(t,"%s ",kb2str(fs->write_rate));
		t += sprintf(t," %s/",kb2str(fs->free));
		t += sprintf(t,"%s\n",kb2str(fs->blocks));
	}

	if (t + 82 > e) {
		(void) write(STDOUT_FILENO,text,t - text);
		t = text;
	}
	memcpy(t,"================================================================================\n",81);
	t += 81;
	(void) write(STDOUT_FILENO,text,t - text);
}


unsigned display_stdout(sysstat_t *s)
{
	unsigned numdis = 0;
	sysstat_t *f = s;
	if (s == 0)
		return 0;
	do {
		if (s->displayed == 0) {
			display_sysstat(s);
			s->displayed = 1;
			++numdis;
		}
		s = s->next;
	} while (s != f);
	return numdis;
}

// vim:tw=0
